/*

  This is early beginning tutorial, explaining simple window creation using WinAPI
  without any messing up with DirectX/OpenGL stuff.

  LICENSE AGREEMENT

  All the parts of this file are provided AS IS and completely WITHOUT ANY WARRANTY OF ANY KIND,
  AND WITHOUT ANY RESPONSIBILITY OF ANY KIND.

  In no event shall author be liable for any indirect, incidental or any other damages (Including
  loss of data, loss of any information, loss of any profit of any kind, loss of any business of any kind,
  and the like), even if arised from any error or any defect in this file.

  Products, trademark names and company names may be appearing in this file may or may not be registered
  trademarks or copyrights of their respective companies or owners of any kind.

*/


#include <windows.h>

// Declare some variables and function prototypes for further usage

HDC dc;

HWND w_handle;
HINSTANCE inst;

LRESULT CALLBACK w_process(HWND wnd_handle,UINT system_msg,WPARAM w_param,LPARAM l_param);

WPARAM _process(HWND w_handle);

// Below functions we need

// Main function
int WINAPI WinMain(HINSTANCE inst,HINSTANCE prev_inst,LPSTR line,int show)
{
	// We want window styles at first

	DWORD ex_style;
	DWORD style;

	// We 'll need it later to set up window size and position.

	RECT rect;
	
	// And at last we need to set a window class

	WNDCLASSEX w_class;

	// Allocate memory for class structure

    memset(&w_class,0,sizeof(WNDCLASSEX));

	// Set up class variables below

	w_class.cbClsExtra = 0; // extra bytes, we want it to be 0
	w_class.cbSize = sizeof(WNDCLASSEX); // size of the whole structure
	w_class.cbWndExtra = 0; // yet another extra bytes, shulda be 0 in our case
	w_class.hbrBackground = (HBRUSH)(COLOR_WINDOW+2); // set up background color, it 'll be black. 
	//if you want it to be white, set +1 instead +2 then
	w_class.hCursor = LoadCursor(NULL,IDC_ARROW); // we 'll set standard cursor, but you can set your own one
    w_class.hIcon = LoadIcon(NULL,IDI_APPLICATION); // standard icon, but culda be NULL instead of it also
    w_class.hIconSm = LoadIcon(NULL,IDI_WINLOGO); // same as above
	w_class.hInstance = inst; // handle to instance
	w_class.lpfnWndProc = w_process; // pointer to window procedure
	w_class.lpszClassName = "test"; // class name
	w_class.lpszMenuName = NULL; // menu name, but if set to NULL, window have no default menu then
	w_class.style = CS_HREDRAW | CS_VREDRAW; // set class style(s)

	// After class infromation is OK and available, it's time to register the class

	RegisterClassEx(&w_class);
	
	// Window styles below

	style = WS_OVERLAPPEDWINDOW;
	ex_style = WS_EX_APPWINDOW;

	// Window rect to set window size (width and height), and window position (left and top)
	// Note that 0,0 yields a top left corner of the screen

	rect.bottom = 480; 
	rect.left = 0;
	rect.right = 640; 
	rect.top = 0;

	// Before creating the window we also need to set up window rectangle and style
 
	AdjustWindowRect(&rect,style,FALSE); // This one only valid before window is created, to change rect of existing window
	// you shulda be using SetWindowPos function from WinAPI

	// Ok, we're ready to create it

	w_handle = CreateWindowEx(
	ex_style, // window style (extended), we had to set up before
	"test", // class name
	"Test", // window name
	style, // yet another window style
	0, // horisontal window position
	0, // vertical window position
	rect.right - rect.left, // window width
	rect.bottom - rect.top, // window height
	NULL, // handle for parent window, it's NULL 'coz we have no such window
	NULL, // handle for menu, it's NULL as well
	inst, // handle to instance
	NULL); // Additional info, but we don't want it at all, so it's NULL too

	// Now let's set window position as well, but note it ain't really necessary, if we 'll skip this, window 'll
	// be applied to the top left of screen

	SetWindowPos(      
    w_handle, // window handle
    HWND_TOP, // how it shulda behave, we wanna it on the top of
    240, // where the left side shulda be
    320, // same for top, note both variables given in client coords
    NULL, // specify new width of the window
    NULL, // same for height, this one and above shulda be given in pixels, but note we don't want it that way
    SWP_NOSIZE); // 'coz we want 2 items we have above to bee ignored, we set NOSIZE to remain window as it is

	// We now want to set up window show state

	ShowWindow(w_handle,SW_SHOW); // window handle and yet another param to make our window active

	ShowCursor(true); // show mouse cursor too, if no longer required it culda be set to FALSE then

	UpdateWindow(w_handle); // update window

	return _process(w_handle); // return result back to, after it's handled by _process function
}

// Shulda be declared within the class in WinMain function
LRESULT CALLBACK w_process(HWND w_handle,UINT _msg,WPARAM wparam,LPARAM lparam)
{
	LONG lret = 0;

	// Below we need to handle system messages, but we only take care about WM_CLOSE

	switch(_msg)
	{

	case WM_CLOSE : // Exit from application if WM_CLOSE command incoming
	{

	ReleaseDC(w_handle,dc); // Release device context from memory

	UnregisterClass("Test",inst); // Remove our class too

	PostQuitMessage(0); // Kill application
	}
	break;

    default : 

	lret = DefWindowProc(w_handle,_msg,wparam,lparam); // Default processing for all the messages not processed by
	// our application, to make sure that all the messages are processed anyway.

	break;
	}

	return lret;
}

// We will pass window handle to this one in WinMain function (at the end of)
WPARAM _process(HWND w_handle)
{
	MSG msg;

	// This one is the main application loop, it's infinite.

	while(1)										
	{		
	if(PeekMessage(&msg,NULL,0,0,PM_REMOVE)) // Dispatch system messages
    { 
	if(msg.message == WM_QUIT) break; // If got WM_QUIT message then break the loop and exit from application

    TranslateMessage(&msg);	// Otherwise, let's find out what the message is exactly				
    DispatchMessage(&msg); // And execute it

	// Here we culda have an else statement and some stuff inside of it, for example, scene rendering, but we leave
	// it as is, 'coz we have nothing to do at this time.

    }
	}

	return(msg.wParam);	
}
